function [sp] = Coffset(cu,d)
//kn is the knot vector
//coefs are the control points
//cu is the 2D curve
//d is the amount of offset

// Display warning for floating point exception
ieee(1);

//Assumption is that the curve is in x,y plane and therefore direction is not important.

//Ouput variables initialisation (not found in input variables)
sp=[];
[row n] = size(cu.coefs);
//right now its for a simple curve and "n" is the number of control points.
u=max(3,n);
k = 2 * u;
//k is the number of control points that we will sample for the offset curve.
limit = 4;
//limit is the number of times parametrization has to be refined.
err = .01
//error we want to allow

//Below is an array of the sample points
//Xd(ti) = X(ti) + d * ncap(ti)

deg = cu.order;
deg = deg - 1;

tempcu = cu;
//We want curves in the form of [x,y] only. if the dimension is 1 then make it 2 by setting x = t;
if row==1 then
	tempcoef1 = [];
	tempcoef2 = cu.coefs;
	iter_count = n; 
	for j=1;1;iter_count
		temp = 0;
		for k=j+1;1;j+deg-1
			temp = temp + tempcu.coefs(k);
		end;
		temp = temp/deg;
		tempcoef1(j) = temp;
	end;
	tempcu.coefs = [tempcoef1; tempcoef2];
end;	

Tvec = [];
for i=0:1:k
	t = i/k;
	Tvec(i+1) = t;
	val = fnval(cu,t);
	normal = Cnormal(cu,t);
	dval = val + d * normal;
	P(1,i+1) = dval(1,1);
	P(2,i+1) = dval(2,1);	
end;

J = 1;
W0 = P(:,1) 
W3 = P(:,k+1)
 
//Solving the optimization equations of lambda1 and lambda2 below
v10 = tempcu.coefs(:,1) - tempcu.coefs(:,2);
vn1n2 = tempcu.coefs(:,n) - tempcu.coefs(:,n-1);

aa = 0;
bb = 0;
ab = 0;
ad = 0;
bd = 0;

for i=1:1:k+1
	di = P(:,i);
	t = Tvec(i);
	b30 = W0 * Bcompute(3,0,t);
	b31 = W0 * Bcompute(3,1,t);
	b32 = W3 * Bcompute(3,2,t);
	b33 = W3 * Bcompute(3,3,t);
	di = di - b30 - b31 - b32 - b33;
	ai = v10 * Bcompute(3,1,t);
	bi = vn1n2 * Bcompute(3,2,t);
	aa = aa + ai' * ai;
	bb = bb + bi' * bi;
	ab = ab + ai' * bi;
	ad = ad + ai' * di;
	bd = bd + bi' * di
end;

//find x such that Ax = b
A = [aa ab; ab bb];
b = [ad; bd];
	
lambda = Solve_linear(A,b);
W1 = W0 + lambda(1) * v10;
W2 = W3 + lambda(2) * vn1n2;
//Now we have W0, W1, W2, W3 which are the control polygons for the curve;
a = Tvec(1);
b = Tvec(k+1);
nots = [a a a a b b b b];
//its a degree three bezier curve
approxC = spmak(nots,[W0 W1 W2 W3]);

d = [];
for i=1:1:k+1
	temp = P(:,i) - fnval(approxC,Tvec(i));
	d(i) = sqrt(temp(1)**2 + temp(2)**2);
end;
dmax = max(d);

newTvec = []
for i=1:1:k+1
	newTvec(i) = newParameterization(cu,P(:,i),Tvec(i));
end;

sp = approxC;
J = J + 1;


return;
endfunction;
