function [vect] = LCPeval(CP,argument)

// Ouput variables initialisation (not found in input variables)
vect=[];

// Display warning for floating point exception
ieee(1)

//CP has the control polygon structure , i.e., has CP.greville and CP.vector
// if CP.greville is an 1-by-m  matrix then CP.vector is dim-by-m.
// CP is the control polygon of a curve 
// dim is the ambient model space dimension
// argument is a real number and output vect is a column
// 
// LCPeval evaluates the control polygon CP at argument

 minii = CP.greville(1);
 maxii = max(CP.greville);

TOL = 10^(-6);

if abs(argument - minii) < TOL then
  argument = minii;
end;

if abs(argument - maxii) < TOL then
  argument = maxii;
end;

if (argument < minii) | (argument > maxii) then
  error("argument outside greville range");
end;

[ig,m] = size(CP.greville);

ii = LLocate(argument, CP.greville);
//LLocate locates ii such that CP.greville(ii)<=argument bu CP.greville(ii+1)>argument
if ii == m then
  vect = CP.vector(:,m);  return;
end;

initial = CP.vector(:,ii);
final = CP.vector(:,(ii+1));

vect=(argument-CP.greville(ii))*final +(CP.greville(ii+1)-argument)*initial;
vect=vect/(CP.greville(ii+1)-CP.greville(ii));

return;
//LCPeval used in knot-insertion etc.
endfunction
